/**
 * \file schemeeditor.cpp
 * A dialog where you can edit tagging schemes.
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 11 Aug 2011
 *
 * Copyright (C) 2011  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "schemeeditor.h"
#ifdef HAVE_QT5
#include <QtWidgets>
#else
#include <QtGui>
#endif
#include "qoobarglobals.h"
#include "taggingscheme.h"
#include "applicationpaths.h"

#include "enums.h"

SchemeEditor::SchemeEditor(const QString &file, Operation operation, QWidget *parent) :
    QDialog(parent)
{DD

    setWindowTitle(tr("Scheme editor"));

    m_operation = operation;

    scheme = new TaggingScheme(file);
    scheme->read();
    edit = new QLineEdit(this);

    if (m_operation==Modify) {
        edit->setText(scheme->name());
        edit->setReadOnly(true);
        fileName = file;
    }
    else if (m_operation==Copy) {
        static int newSchemeIndex = 1;
        edit->setText(QString("%1 (%2)").arg(scheme->name()).arg(newSchemeIndex));
        newSchemeIndex++;
    }

    table = new QTableWidget(TAGSCOUNT,5,this);
    table->setHorizontalHeaderLabels(QStringList()<<"ID3v2"
                                     <<"APE"<<"Vorbis"<<"MP4"<<"ASF");
    for (int tagID=0; tagID<TAGSCOUNT; ++tagID) {
        for (int tagType=0; tagType<5; ++tagType) {
            QString text = scheme->fields(tagID,(TaggingScheme::TagType)tagType).join(";");
            QTableWidgetItem *item = new QTableWidgetItem(text);
            if (text=="internal") item->setFlags(Qt::ItemIsSelectable);
            table->setItem(tagID,tagType,item);
        }
        table->setVerticalHeaderItem(tagID,new QTableWidgetItem(Qoobar::tagByID(tagID)));
    }

    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Save |
                                                       QDialogButtonBox::Cancel);
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(edit);
    layout->addWidget(table);
    layout->addWidget(buttonBox);
    setLayout(layout);
    edit->selectAll();

    resize(750,410);
}

SchemeEditor::~SchemeEditor()
{DD

    delete scheme;
}

void SchemeEditor::accept()
{DD

    QString text = edit->text();
    scheme->setName(text);
    if (m_operation==Copy)
        scheme->setFilePath(ApplicationPaths::userSchemesPath() + "/" + text + ".xml");
    //writing of scheme
    scheme->clear();
    for (int j=0; j<5; ++j) {
        for (int i=0; i<TAGSCOUNT; ++i) {
            scheme->setFields(i,(TaggingScheme::TagType)j, table->item(i,j)->text().split(";"));
        }
    }

    if (scheme->save())
        QDialog::accept();
    else QDialog::reject();
}
